﻿using System;
using System.Collections.Generic;
using ELTE.ExamGenerator.Model;
using ELTE.ExamGenerator.View;
using ELTE.ExamGenerator.ViewModel;
using Xamarin.Forms;

namespace ELTE.ExamGenerator
{
    public class App : Application
    {
        private IExamGeneratorModel _model;
        private ExamGeneratorViewModel _viewModel;

        public App()
        {
            MainPage = new MainPage();
        }

        protected override void OnStart()
        {
            OnResume();
        }

        protected override void OnSleep()
        {
            Properties["questionCount"] = _model.QuestionCount; // elmentjük a tételek számát
            Properties["periodCount"] = _model.PeriodCount;

            // lekérdezzük a nem választható tételeket
            List<Int32> takenNumbers = new List<Int32>();
            for (Int32 number = 1; number <= _model.QuestionCount; number++)
            {
                if (!_model.IsTakeable(number))
                    takenNumbers.Add(number);
            }

            Properties["takenNumbers"] = takenNumbers; // elmentjük a nem választható tételeket

            if (_model.IsGenerating) // ha fut a generálás
            {
                Properties["wasGenerating"] = true; // akkor ezt elmentjük
                _model.Take(); // leállítjuk a generálást
            }
            else
            {
                Properties["wasGenerating"] = false;
            }

            _model = null;
            _viewModel = null;
			MainPage.BindingContext = null;
			// a szemétgyűjtő kitörli a memóriából a modellt és a nézetmodellt
        }

        protected override void OnResume()
        {
            // ha el lett mentve az állapot, akkor visszatöltjük
            if (Properties.ContainsKey("questionCount"))
            {
                _model = new ExamGeneratorModel((Int32)Properties["questionCount"], (Int32)Properties["periodCount"]);
                // az elmentett adatokkal példányosítjuk a modellt
            }
            else
            {
                _model = new ExamGeneratorModel(10, 0);
                // alapbeállítás a modellre
            }
            
            if (Properties.ContainsKey("takenNumbers"))
            {
                List<Int32> takenNumbers = (Properties["takenNumbers"] as List<Int32>);

                for (Int32 number = 1; number <= _model.QuestionCount; number++)
                {
                    _model.SetTakeable(number, !takenNumbers.Contains(number));
                }
            }

            _viewModel = new ExamGeneratorViewModel(_model);

            MainPage.BindingContext = _viewModel; // nézetmodell befecskendezése
            
            // ha korábban futott a generálás, folytatjuk
            if (Properties.ContainsKey("wasGenerating") && (Boolean)Properties["wasGenerating"] == true)
            {
                _model.Generate();
            }
        }
    }
}
